#!/bin/sh

POWER_THRESHOLD=170
MONITOR_INTERVAL=1

function SENDCOMMAND() {
  local lRES;

  lRES=`ipmitool -b 6 -t 0x88 raw $* 2>&1`
  if [ 0 -eq $? ]; then
    echo ${lRES}
    return 0
  else
    lRES=`echo ${lRES} | awk '{for(i=1; i<=NF; i++) if($i ~ /^rsp=0x/) {print substr($i, 5, 4);}}'`
    echo ${lRES}
  fi
  return 1
}

case $1 in
  "monitor" ) 
    CAPPING="UNKNOWN"
    while :
    do
      RES=`ipmitool sensor get "Power Unit" 2>NUL`
      if [ 0 -eq $? ]; then
        STATE=`echo ${RES} | grep "AC lost"`
        if [ "" != "${STATE}" ]; then
          if [ "ENABLE" != ${CAPPING} ]; then
            ERR=0
            RES=`SENDCOMMAND 0x2E 0xC0 0x57 0x01 0x00 0x04 0x00 0x0B`
            if [ 0 -ne $? ]; then
              ERR=$((ERR+1))
            fi
            LIMITHI=$((${POWER_THRESHOLD}/256))
            LIMITLO=$((${POWER_THRESHOLD}%256))
            RES=`SENDCOMMAND 0x2E 0xC1 0x57 0x01 0x00 0x10 0x0B 0x50 0x01 ${LIMITLO} ${LIMITHI} 0x70 0x17 0x00 0x00 0x00 0x00 0x10 0x0E`
            if [ 0 -ne $? ]; then
              ERR=$((ERR+2))
            fi
            RES=`SENDCOMMAND 0x2E 0xC0 0x57 0x01 0x00 0x03 0x00 0x0B`
            if [ 0 -ne $? ]; then
              ERR=$((ERR+4))
            fi
            RES=`SENDCOMMAND 0x2E 0xC0 0x57 0x01 0x00 0x01 0x00 0x0B`
            if [ 0 -ne $? ]; then
              ERR=$((ERR+8))
            fi
            if [ 0 -eq ${ERR} ]; then
              echo "Enable power capping. PowerThreshold="${POWER_THRESHOLD}"w"
              CAPPING="ENABLE"
            fi
          fi
        else
          if [ "DISABLE" != ${CAPPING} ]; then
            RES=`SENDCOMMAND 0x2E 0xC0 0x57 0x01 0x00 0x04 0x00 0x0B`
            if [ 0 -eq $? ]; then
              echo "Disable power capping."
              CAPPING="DISABLE"
            fi
          fi
        fi
        sleep ${MONITOR_INTERVAL}
      fi
    done
  ;;

  "disable") 
    RES=`SENDCOMMAND 0x2E 0xC0 0x57 0x01 0x00 0x04 0x00 0x0B`
    if [ 0 -eq $? ]; then
      echo "Disable power capping."
    else
      case ${RES} in
        0x80 ) 
          echo "Power capping is disabled." ${RES}
        ;;
        *)
          echo "Command failed. please try again." ${RES}
          exit 1
        ;;
      esac
    fi
  ;;

  "show" )
    RES=`SENDCOMMAND 0x2E 0xC2 0x57 0x01 0x00 0x00 0x0B`
    if [ 0 -eq $? ]; then
      ENABLED=`echo ${RES} | awk '{ print $4 }'`
      if [ 0 -ne $((0x${ENABLED} & 0x10)) ]; then
        THRESHOLD=`echo ${RES} | awk '{ print $8$7 }'`
        THRESHOLD=`printf "%d" 0x${THRESHOLD}`
        echo "Power capping is enabled. PowerThreshold="${THRESHOLD}"w"
      else
        echo "Power capping is disabled."
      fi
    else
      case ${RES} in
        0x80 ) 
          echo "Power capping is disabled." ${RES}
        ;;
        *)
          echo "Command failed. please try again." ${RES}
          exit 1
        ;;
      esac
    fi
  ;;

  * )
    echo "Invalid option!" $1
    echo "Usage: $0 {monitor|disable|show}"
  ;;
esac

exit 0
